# -*- coding: utf-8 -*-
"""
Created on Tue May 24 14:15:13 2022

@author: sjmvanoosterom
"""
import gurobipy as grb
import numpy as np
import datetime as dt

from math import radians, degrees, sin, cos, asin, acos, sqrt

def Great_circle_distance(lon1, lat1, lon2, lat2):
    lon1, lat1, lon2, lat2 = map(radians, [lon1, lat1, lon2, lat2])    
    return 6371 * ( acos(sin(lat1) * sin(lat2) + cos(lat1) * cos(lat2) * cos(lon1 - lon2))  )

def clear_line(n=1):
    LINE_UP = '\033[1A'
    LINE_CLEAR = '\x1b[2K'
    for i in range(n):
        print(LINE_UP, end=LINE_CLEAR)

class Airport:
    def __init__(self, name , IATA , ICAO, lat , long):
        self.name = name
        self.IATA = IATA
        self.ICAO = ICAO
        self.lat  = lat
        self.long = long
    
    def dist(self,other):
        return Great_circle_distance(self.lat, self.long , other.lat, other.long)
    
    def __le__(self,other):
        return self.IATA <= other.IATA
    
    def __lt__(self,other):
        return self.IATA < other.IATA
    
    def __repr__(self):
        return self.name

class Departure:
    def __init__(self, t_d, ac):
        self.t_d    = t_d
        self.ac     = ac
        
    def __repr__(self):
        return self.ac + ' departure at ' + str(self.t_d)
    
    def __le__(self, other ):
        return self.t_d <= other.t_d
    
    def __lt__(self,other):
        return self.t_d < other.t_d

class Battery:
    
    def __init__(self , t_a , t_c, ac):
        self.t_a    = t_a
        self.t_c    = t_c 
        self.ac     = ac
        self.name   = ac + '('+ t_a.strftime("%H.%M")+')'
        self.I      = []
        
    def add_interval(self, i):
        self.I.append(i)
        
    def __repr__(self):
        return self.ac + ' arrival at ' + str(self.t_a) + ', tc = ' + str(np.round(self.t_c/3600,2))
        
    def __lt__(self,other):
        return self.t_a < other.t_a
    
    def __le__(self,other):
        return self.t_a <= other.t_a
        
class Time_Interval:
    
    def __init__(self, t_s , t_e , c_e):
        self.t_s    = t_s 
        self.t_e    = t_e 
        self.t      = (t_e - t_s).seconds
        self.c_e    = c_e 
        self.B      = []
        self.Dem    = 0
        self.name   = t_s.strftime("%H.%M")
        
        self.I_previous = []
    
    def add_bat(self, b):
        self.B.append(b) 
        
    def __repr__(self):
        return self.t_s.strftime("%H.%M") + '-'+ self.t_e.strftime("%H.%M")
        
        

class E_AC_Charge_Schedule_Instance:
    
    def __init__(self, start_of_day,  bats , departures, n_s , n_c, n_night , C_e , c_d):
        self.bats   = bats
        self.deps   = departures
        self.n_s    = n_s
        self.n_c    = n_c    
        self.n_n    = n_night
        self.c_d    = c_d
        self.c_d_com= c_d
        self.C_e    = C_e 
        
        self.bats.sort()
        self.deps.sort()
        
        self.start_of_day   = start_of_day
        
        (d_SRTP , e_SRTP,  t_max)     = self.Solve_Model_SRPT()
        # print('SRPT delay: ', int(d_SRTP))     
        # print('SRPT elec:  ', int(e_SRTP))  
        
        self.delay_cost_SRTP= d_SRTP
        self.elec_cost_SRTP = e_SRTP

        # self.end_of_day     = max(start_of_day + dt.timedelta(days = 1),
        #                           t_max + dt.timedelta(hours = 1))
        if t_max >= start_of_day + dt.timedelta(days = 1, hours = 10):
            self.end_of_day = t_max
        else:
            self.end_of_day = start_of_day + dt.timedelta(days = 1, hours = 1)
        
        self.Intervals   = self.Add_intervals()
        
        # calculate minimum electricity price
        self.elec_cost_min  = sum(b.t_c for b in self.bats) * min([c[2] for c in C_e])
        self.elec_cost_max  = sum(b.t_c for b in self.bats) * max([c[2] for c in C_e])
        # initialize the model 
        
        self.model = grb.Model('E_AC_Infra_Model')
        
        # add variables 
        self.var_T_c         = {}
        self.var_Rdy         = {}
        self.var_Dem_min_Sup = {}

        for b in self.bats:
            for i in b.I:
                self.var_T_c[(b,i)] = self.model.addVar(vtype = grb.GRB.CONTINUOUS , lb = 0 , ub = i.t , name = 't_c_'+ str(b.name) +'_'+str(i.name))
                self.var_Rdy[(b,i)] = self.model.addVar(vtype = grb.GRB.BINARY     , name = 'Rdy_'+ str(b.name) +'_'+str(i.name))
        
        for i in self.Intervals:
            self.var_Dem_min_Sup[i] = self.model.addVar(vtype = grb.GRB.INTEGER , lb = 0, name = 'dem_min_sup_'+ i.name)
    
    def __repr__(self):
        return 'Charge_Shedule_Opt at  , , ns = '+ str(self.n_s) + ', nc= '+str(self.n_c)

    def Add_intervals(self):
        
        dT = self.end_of_day - self.start_of_day
        dT = dT.days * 24 * 3600 + dT.seconds
        
        tims = np.arange(0, dT + 1 , 15 * 60)
        
        T = [self.start_of_day + dt.timedelta(seconds = float(t)) for t in tims]
        
        I = []
        
        for t in range(len(T)-1):
            #print(T[t], end = ', ')
            # determine c_e

            for cost_interval in self.C_e:
                if cost_interval[0] <= T[t] and T[t+1] <= cost_interval[1]:
                    c_e = cost_interval[2]

            I0 = Time_Interval(T[t] , T[t+1] , c_e)
            
            for b in self.bats:
                if b.t_a <= T[t]:
                    I0.add_bat(b)
                    b.add_interval(I0)
            for dep in self.deps:
                if dep.t_d <= T[t+1]:
                    I0.Dem += 1
            #print()
            
            I.append(I0)
            I0.I_previous   = [i for i in I]
            
        return I
    
    def Add_Constraints(self):
        
        # constraint 1, define the difference in supply and demand
        self.Constr1 = {}
        for i in self.Intervals:
            self.Constr1[i] = self.model.addConstr(  sum(self.var_Rdy[(b,i)] for b in i.B) + self.var_Dem_min_Sup[i]  >= - self.n_n -  self.n_s + i.Dem,
                                  name = 'C1_'+i.name)
        
        # constraint 2, cumulative use <= total power 
        for i in self.Intervals:
            
            if len(i.B) > 0:
                self.model.addConstr(sum(self.var_T_c[(b,i)] for b in i.B) <= self.n_c * i.t,
                                      name = 'C2_'+ i.name)
            
        # constraint 3, full recharge 
        for b in self.bats:
            self.model.addConstr(sum(self.var_T_c[(b,i)] for i in b.I) >= b.t_c, 
                                 name = 'C3_' + b.name)
            
        # constraint 4, only ready when fully charged 
        for b in self.bats:
            for i in b.I:
                intervals = list(set(i.I_previous).intersection(set(b.I)))
                self.model.addConstr(self.var_Rdy[(b,i)] <= 
                                      1 - (b.t_c - sum(self.var_T_c[(b,ip)] for ip in intervals)) / b.t_c)
        
        
    def Update_Spare_Bats(self, n_s):
        self.n_s = n_s
        
        for i in self.Constr1:
            self.Constr1[i].rhs = i.Dem - self.n_n - self.n_s
            
        self.model.update()
    
    def Add_Objective(self):
        self.model.setObjective(sum(sum(self.var_Dem_min_Sup[i-int(t/15)] * self.c_d[t] for t in self.c_d) for i in self.Intervals) +
                                sum(self.var_T_c[(b,i)] * i.c_e for (b,i) in self.var_T_c) )
        
    def Optimize(self, write = 0):
        #self.model.write('test.lp')
        if write == 0:
            self.model.setParam('LogToConsole', 0)
        else:
            self.model.setParam('LogToConsole', 1)
        self.model.setParam('TimeLimit',60)
        self.model.setParam('MIPgap',0.01)
        self.model.optimize()
        
        
        # ===============================return results=========================================
        
        if self.model.Status == 3:
            return 'Infeasible'
        elif self.model.Status == 2 or self.model.Status == 9:
            T_c = {}
            Rdy = {}
            for (b,i) in self.var_T_c:
                T_c[(b,i)] = self.var_T_c[(b,i)].X
                intervals = list(set(i.I_previous).intersection(set(b.I)))
                if sum(self.var_T_c[(b,ip)].X for ip in intervals) >= 0.98 * b.t_c:
                    Rdy[(b,i)] = 1
                else:
                    Rdy[(b,i)] = 0
                #Rdy[(b,i)] = np.floor(1 - np.round(b.t_c - sum(self.var_T_c[(b,ip)].X for ip in intervals) / b.t_c ,3) )
                
            Dem_min_sup = {}
            for x in self.var_Dem_min_Sup:
                Dem_min_sup[x] = self.var_Dem_min_Sup[x].X
                
            Tot_chr_time = {i:0 for i in self.Intervals}
            for i in self.Intervals:
                for b in i.B:
                    Tot_chr_time[i] += self.var_T_c[(b,i)].X
                    
            tot_cost = self.c_d * sum(Dem_min_sup[i] for i in Dem_min_sup) + sum(Tot_chr_time[i] * i.c_e for i in self.Intervals)
                
            # split the total cost in delay, electricity, costs

            # delay cost accounting for the nonlinear cost function
            #  dem - dem_min_sup -> sup -> chr times -> sort -> sum char[i] - dem[i] over costs
            Supply = [self.start_of_day] * (self.n_s + self.n_n) 
            for b in self.bats:
                t_c_min = self.start_of_day + dt.timedelta(days = 10)
                for i in b.I:
                    t_c_min = min(t_c_min , i.t_e + dt.timedelta(days = 10) * (1-Rdy[(b,i)]))
                Supply.append(t_c_min)
            Supply.sort()

            Demand = [d.t_d for d in self.deps]
            Demand.sort()

            cost_delay = 0
            for i in range(len(Demand)):
                for t in self.c_d_com:
                    cost_delay += max(0, (Supply[i] - Demand[i]).total_seconds() / 60 - t ) * self.c_d_com[t] # minutes of delay * cost per minute
        

            #cost_delay = self.c_d * sum(Dem_min_sup[i] for i in Dem_min_sup)
            cost_elec = sum(Tot_chr_time[i] * i.c_e for i in self.Intervals)
            
            return (T_c , Rdy , Dem_min_sup , Tot_chr_time , cost_delay, cost_elec)
    
    def Solve_Model(self, write = 0):
        self.Add_Constraints()
        self.Add_Objective()
        
        return self.Optimize(write)
    
    def Solve_Model_SRPT(self):
        # use the SRPT approach to determine when to charge which battery
        
        dift        = dt.timedelta(seconds = 900)
        t           = self.start_of_day
        bats        = {b:{'tc':b.t_c, 'charged': False} for b in self.bats}
        not_charged = len(self.bats)
        sup         = [self.start_of_day] * (self.n_s + self.n_n)
        elec_cost   = 0
        
        
        while not_charged > 0:
            for cost_interval in self.C_e:
                if cost_interval[0] <= t and t <= cost_interval[1]:
                    c_e = cost_interval[2]
            
            av_bats1 = {b for b in self.bats if b.t_a <= t}
            av_bats2 = {b for b in bats if bats[b]['charged'] == False}
            
            av_bats = av_bats1.intersection(av_bats2)
            ab = [(bats[b]['tc'], b) for b in av_bats]
            ab.sort()
            # print(t)
            # print('len ab = ', len(ab))
            # print(ab)
            if self.n_c >= len(ab):
                for (tc,b) in ab:
                    bats[b]['tc'] += -dift.seconds
                    elec_cost   += (dift.seconds) * c_e
                    if bats[b]['tc'] <= 0:
                        bats[b]['charged']  = True
                        not_charged += -1
                        sup.append(t + dift)
                        
            else:
                for (tc,b) in ab[:int(np.floor(self.n_c))]:
                    bats[b]['tc'] += -dift.seconds
                    elec_cost   += (dift.seconds ) * c_e
                   # print('charging', b.name ,' at ', t, ' remaining tc = ', bats[b]['tc'])
                    if bats[b]['tc'] <= 0:
                        bats[b]['charged']  = True
                        not_charged += -1
                        sup.append(t + dift)
                        
                if int(np.floor(self.n_c)) != self.n_c:
                    tc, b = ab[int(np.floor(self.n_c))]
                    bats[b]['tc'] += - float(dift.seconds) * float(self.n_c%1)
                    elec_cost   += (float(dift.seconds) * float(self.n_c%1) ) * c_e
                    if bats[b]['tc'] <= 0:
                        bats[b]['charged']  = True
                        not_charged += -1
                        sup.append(t + dift)
            
            t += dift
        
        # determine the delay
        # print('Last time: ', t)
        dep = [d.t_d for d in self.deps]
        dep.sort()
        # print(sup)
        # print(dep)
        c_delay = 0
        for i in range(len(dep)):
            # print(dep[i], sup[i])
            for t_del in self.c_d_com:
                c_delay += max(0, (sup[i] - dep[i]).total_seconds() / 60 - t_del ) * self.c_d_com[t_del] # minutes of delay * cost per minute
        
        return (c_delay  , elec_cost , t)
        