#%% -*- coding: utf-8 -*-
"""
Created on Tue May 24 14:42:49 2022

@author: sjmvanoosterom
""" 

import E_AC_Infra_Object_Classes  as OBJ 
import E_AC_Infra_Background_Functions as BF
import datetime as dt
import numpy as np
import pickle
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from IPython.display import clear_output

plt.rcParams.update({
    "font.family": "serif"
})

#%% DEFINE THE INPUT PARAMETERS OF THE PROBLEM

airports_file   = 'input_data/GlobalAirportDatabase.txt'
flights_folder  = 'input_data/Flight_schedules_wideroe'

npax            = 10 # passengers in the Alice
npaxnom         = 30  # passengers in normal aircraft
Pch             = 200 # kW charging power
C               = 900 # kWh battery capacity

c_delay         = {0 : 10 , 45 :  9.9 ,120 :  11} #  €/minute within given intervals
c_spare         = 250  # €/sparebat/day to operate and maintain ; based on specs from alice 
c_charger       = 40 *Pch / 100 # €/charger/day to operate and maintain ; based on Justin ; Pch in kW
c_e_peak        = 0.134 # €/kWh, norwegian energy price
c_e_off         = 0.116 # €/kWh, off peak norwegian energy price

c_e_peak        = c_e_peak * Pch / 3600 # €/s charging price on peak power at peak hours
c_e_off         = c_e_off  * Pch / 3600 # €/s charging price on peak power at off peak hours

fl_range        = 800 * 0.85 # km
max_ch_time     = 8 # hours to charge to full battery


#%% RETRIEVE THE FLIGHT DATA

all_airports = BF.Read_airports(airports_file)

flights , airports = BF.Read_flight_Schedule(flights_folder , all_airports)

#PLOT THE BUSIEST AIRPORTS
BF.PlotMostPopAps(flights)
plt.savefig('Output/Wideroe_top_hubs.pdf',bbox_inches = "tight")


#%% CONVERT THE FLIGHTS TO BATTERY CHARGING TASKS

#DEFINE THE HUB AIRPORTS
hubs = [airports['BOO'], airports['TRD'], airports['TOS'] , airports['BGO'], airports['VDS'], airports['OSL'], airports['HFT']]

batteries = BF.Conv_Flights_To_Batteries(flights , hubs , airports, fl_range, C , Pch)

#GRAPH THE FLIGHTS
BF.PLot_Flights(flights , batteries)
plt.savefig('Output/Wideroe_network.pdf', bbox_inches = 'tight')



#%% SORT THE TASKS INTO A PER DAY PER AIRPORT FORMAT

sb = BF.Sort_batteries_per_day_and_airport(batteries, hubs)
BF.PlotInOutHist(sb)


# CALCULATE THE AVERAGE DAILY ENERGY DEMAND PER HUB
print('Average energy demand and number of flights per hub:')
for hub in hubs:
    charge_time = sum( [ sum([bat[4] / fl_range * C / Pch for bat in sb[hub][date] if bat[3][3] == hub ]) for date in sb[hub] ])
    n_flights = sum([len(sb[hub][date]) for date in sb[hub] ] )   
    print(hub , int(charge_time *Pch / 365), round(n_flights/len(sb[hub]),1))


#%% RETRIEVE THE CHARACTERISTICS OF THE DAILY TRAFFIC AT EACH AIRPORT

for i in range(len(hubs)):
    print(i, ': ', hubs[i].IATA, end = ', ')
hub_selec = int(input('Select hub number: '))

hub = hubs[hub_selec]

# 1 SELECT WHICH METHOD IS USED
day_selec = input("Day selection [r]ecourse, [p]eak, [m]edian: ")

# 2 SAMPLE DAYS
N_dates = 30
dates_SA, dates_PD, dates_MD  = BF.Day_Selector(sb , hub , N_dates)
dates_SA.sort()
if day_selec == 'r':
    print('SP Date selection')
    # 1b SP
    dates = dates_SA
elif day_selec =='p':
    # 1c Peak day
    print('PEAK date selection')
    dates = dates_PD
elif day_selec =='m':
    # 1d Average day
    print('Median date selection')
    dates = dates_MD 

# 3 STORE THE DAYS SAMPLE
NoFlightsPerDay = np.array([len(sb[hub][day]) for day in sb[hub]])
print('Flight numbers:  maximum', max(NoFlightsPerDay),
      ' average: ', np.mean(NoFlightsPerDay),
      ' median: ', np.median(NoFlightsPerDay),
      ' std: ', np.std(NoFlightsPerDay))

Nc = np.round(np.arange(1,5.1,0.25), decimals = 2)
Ns = range(1,5)

# 4 INITIALIZE ARRAY WITH COSTS (#CHARGERS X #SPARE_BATTERIES X #DAYS)
COST        = np.zeros((len(Nc), len(Ns), len(dates)))      # TOTAL COSTS
COST_elec   = np.zeros((len(Nc), len(Ns), len(dates)))      # ELECTRICITY COSTS
COST_delay  = np.zeros((len(Nc), len(Ns), len(dates)))      # DELAY COSTS

start_time  = 4 # AM , start time of the day of operations

# 5 IF THE EXPERIMENT WAS PREVIOUSLY PERFORMED, WE RETRIEVE THE STORED DATA
try:
    with open('input_data/' + hub.IATA + '.pkl' , 'rb') as f:
        known_cost = pickle.load(f)
except:
    known_cost = {}

#%% INITIAL SEARCH, ITERATE OVER A BROAD NET OF CHARGERS/SPARE_BATTERIES WITH SMALL NUMBER OF DAYS

for i_d in range(len(dates)):
    date = dates[i_d]
    
    for i_c in range(len(Nc)):
        for i_s in range(len(Ns)):
            n_c = Nc[i_c]
            n_s = Ns[i_s]
            
            print('date = ', i_d, date, ' nc = ',n_c,' ns = ', n_s)
            if (date, n_s, n_c) in known_cost:
                # THE SOLUTION WAS OBTAINED IN PREVIOUS SIMULATIONS
                print()
                print('known')
                COST[i_c, i_s, i_d] = known_cost[(date,n_s,n_c)]
            else:
                # THE SOLUTION WAS NOT OBTAINED IN PREVIOUS SIMULATIONS
                cost_charger = n_c * c_charger
                cost_spares = n_s * c_spare
                
                # CREATE SCHEDULING MODEL
                INST = BF.Create_Day_Schedule_Instance(sb, hub , date, 
                                                 n_c , n_s ,
                                               c_delay, fl_range, bat_cap = C, pc_max = Pch , 
                                               c_e_off = c_e_off , c_e_peak = c_e_peak)
                
                if INST.delay_cost_SRTP < 5000:
                    # SOLVE THE SCHEDULING MODEL
                    ret = INST.Solve_Model()
                    if ret != 'Infeasible':
                        T_c = ret[0] ; Rdy = ret[1] ; Dem_min_sup = ret[2] 
                        Tot_chr_time = ret[3] ; cost_delay = ret[4] ;  cost_elec = ret[5] 
                    
                        print()
                    
                    else:
                        print()
                        print('Infeasible')
                else:
                    cost_delay = INST.delay_cost_SRTP
                    cost_elec  = INST.elec_cost_SRTP
                    print('High delay cost')
                
                # PRINT THE RESULTS TO THE CONSOLE
                print('Chargers : ', int(cost_charger))
                print('Spares:    ', int(cost_spares))
                print('electr:    ', int(cost_elec))
                print('delays:    ', int(cost_delay))
                
                print('tot_cost: ',int(cost_charger + cost_spares + cost_elec + cost_delay ))   
                print()
                
                COST[i_c,i_s,i_d]       = int(cost_charger + cost_spares + cost_elec + cost_delay )
                COST_elec[i_c,i_s,i_d]  = int(cost_elec)
                COST_delay[i_c,i_s,i_d] = int(cost_delay )

            #clear_output()
                

# STORE THE OBTAINED RESULTS TO PICKLE FILES
cost = np.mean(COST, axis = 2)
for i_d in range(len(dates)):
    for i_s in range(len(Ns)):
        for i_c in range(len(Nc)):
            day = dates[i_d]
            n_s = int(Ns[i_s])
            n_c = Nc[i_c]
            known_cost[(day, n_s, n_c)] = COST[i_c, i_s,i_d]
f = open('input_data/'+hub.IATA + ".pkl",'wb')
pickle.dump(known_cost,f)
f.close()

# GRAPH THE OBTAINED RESULTS
fig, axs = plt.subplots()
BF.PlotCostImage(axs, Nc, Ns, COST)

C2 = np.mean(COST,axis = 2)
indx = np.where(C2 == np.min(C2))
nc_min = Nc[indx[0][0]]
ns_min = Ns[indx[1][0]]
print('Nc_min = ' , nc_min, ' Ns_min = ', ns_min)

        
#%% OPTIMIZE THE INFRASTRUCTURE WITH AN ADAPTIVE NEIGHBORHOOD SEARCH ALGORITHM

# SELECT THE INITIAL SOLUTION
Nc = [nc_min]
Ns = [ns_min]
Omega = [np.infty]
Omega_elec = [np.infty]
Omega_delay = [np.infty]

# DEFINE THE SIZES OF THE SEARCH STEPS
dNc = [0.5 , 0.25, 0.125]
dNs = [1 , 1 ,0 , 0 , 0]
i_del = 0



fig , axs = plt.subplots(ncols = len(dNc))

# PERFORM THE NEIGHBORHOOD SEARCH
while i_del < len(dNc):
    NcSub = np.linspace(Nc[-1] - dNc[i_del], Nc[-1] + dNc[i_del], 5 )
    if dNs[i_del] > 0:
        NsSub = np.linspace(Ns[-1] - dNs[i_del] , Ns[-1] + dNs[i_del] , 3)
    else:
        NsSub = [Ns[-1]]
        
    print('NcSub = ', NcSub)
    print('NsSub = ', NsSub)
        

    COST        = np.zeros((len(NcSub), len(NsSub), len(dates)))
    COST_elec   = np.zeros((len(NcSub), len(NsSub), len(dates)))
    COST_delay  = np.zeros((len(NcSub), len(NsSub), len(dates)))
    
    
    for i_d in range(len(dates)):
        date = dates[i_d]

        print(i_d, end = ', ')
        for i_s in range(len(NsSub)):
            #print(i_d, end = ', ')
            for i_c in range(len(NcSub)):
                print((i_s, i_c), end = ', ')
                n_s = int(NsSub[i_s])
                n_c = NcSub[i_c]
                
                if (date, n_s, n_c) in known_cost:
                    COST[i_c, i_s, i_d] = known_cost[(date , n_s,n_c)]
                else:
                    #print('i_del = ',i_del, ' id = ', i_d, date, ' nc = ',np.round(n_c,3),' ns = ', n_s)
                    
                    cost_charger = n_c * c_charger
                    cost_spares = n_s * c_spare
                    
                    INST = BF.Create_Day_Schedule_Instance(sb, hub , date, 
                                                 n_c , n_s ,
                                               c_delay, fl_range, bat_cap = C, pc_max = Pch , 
                                               c_e_off = c_e_off , c_e_peak = c_e_peak)
                
                    if INST.delay_cost_SRTP < 5000:
                        ret = INST.Solve_Model()
                        if ret != 'Infeasible':
                            T_c = ret[0] ; Rdy = ret[1] ; Dem_min_sup = ret[2] 
                            Tot_chr_time = ret[3] ; cost_delay = ret[4] ;  cost_elec = ret[5] 
                        
                            #print()
                        
                        else:
                            #print()
                            print('Infeasible')
                    else:
                        cost_delay = INST.delay_cost_SRTP
                        cost_elec  = INST.elec_cost_SRTP
                        print('High delay cost')     

                    COST[i_c, i_s , i_d]        = int(cost_charger + cost_spares + cost_elec + cost_delay )
                    COST_elec[i_c , i_s , i_d]  = int(cost_elec)
                    COST_delay[i_c, i_s, i_d ]  = int(cost_delay )
                    
        print()
        OBJ.clear_lines(8)
                
    cost = np.sum(COST,axis = 2) / len(dates)
    (inc , ins) = np.unravel_index(np.argmin(cost) , cost.shape)
    nc_min = NcSub[inc]
    ns_min = NsSub[ins]
    c = cost[inc,ins]
    
    print(cost)
    
    if c < Omega[-1]:
        print('Improved solution found: nc ', nc_min , ', ns ', ns_min)
        Nc.append(nc_min)
        Ns.append(ns_min)
        Omega.append(c)
        BF.PlotCostImage(axs[i_del], NcSub , NsSub , COST)

    else:
        print('No Improvement found, going smaller')
        i_del += 1
    
    print()
        
    for i_d in range(len(NsSub)):
        for i_c in range(len(NcSub)):
            n_s = int(NsSub[i_s])
            n_c = NcSub[i_c]
            known_cost[(n_s, n_c)] = cost[i_c, i_s]

print('Nc: ',Nc, 'Ns: ',Ns )
#%% PERFORM A FULL ANALYSIS OF THE FOUND SOLUTIONS
dates = dates_SA

# LIST OF OBTAINED SOLUTIONS
#  {HUB: {METHOD: (#CHARGERS, #SPARE_BATTERIES)}} 
Hsolutions = {hubs[3]: {'SA': (2, 3) , 'PD': (1.75 , 4) , 'MD' :(1 , 2)}, # bergen 
              hubs[0]: {'SA': (4.5 , 6) , 'PD': ( 6.25 ,7), 'MD' : (3.75, 3)}, # bodo
              hubs[6]: {'SA': (2.065, 2) , 'PD': (1.75,2)   , 'MD' : (1.2375, 1)}, # hammerfest
              hubs[5]: {'SA': (3.06, 3) , 'PD': (2.7 , 1) , 'MD' :(2.3 , 1)}, # oslo
              hubs[2]: {'SA': (4.0625, 4) , 'PD': (3.6 , 6) , 'MD' :(1.7, 3)}, # tromso
              hubs[1]: {'SA': (4.25, 5) , 'PD': (4.00 , 6) , 'MD' :(2.2, 3)}, # trondheim
              hubs[4]: {'SA': (2, 2) , 'PD': (1.6 , 3) , 'MD' :(1.6, 2)}, # vadso
              }

NcNs = [Hsolutions[hub][i] for i in Hsolutions[hub]]

COST        = {ncs: [] for ncs in NcNs}
COST_elec   = {ncs: [] for ncs in NcNs}
COST_delay  = {ncs: [] for ncs in NcNs}

start_time  = 4 # AM , start time of the day of operations


for i_d in range(len(dates)):
    date = dates[i_d]
    
    for (n_c, n_s) in NcNs:

        
        print('id = ', i_d, date, ' nc = ',n_c,' ns = ', n_s)

        cost_charger = n_c * c_charger
        cost_spares = n_s * c_spare
        
        INST = BF.Create_Day_Schedule_Instance(sb, hub , date, 
                                                 n_c , n_s ,
                                               c_delay , fl_range, 
                                               bat_cap = C, pc_max = Pch , 
                                               c_e_off = c_e_off , c_e_peak = c_e_peak)
        
        if INST.delay_cost_SRTP < 10000: #PRUNE A SOLUTION IF THE COSTS ARE TOO HIGH
            ret = INST.Solve_Model()
            if ret != 'Infeasible':
                T_c = ret[0] ; Rdy = ret[1] ; Dem_min_sup = ret[2] 
                Tot_chr_time = ret[3] ; cost_delay = ret[4] ;  cost_elec = ret[5] 
            else:
                print('Infeasible')
        else:
            cost_delay = INST.delay_cost_SRTP
            cost_elec  = INST.elec_cost_SRTP
        
        # PRINT THE OUTPUT
        clear_output()
        print('id = ', i_d, date, ' nc = ',n_c,' ns = ', n_s)
        print('Chargers : ', int(cost_charger))
        print('Spares:    ', int(cost_spares))
        print('electr:    ', int(cost_elec))
        print('delays:    ', int(cost_delay))
        
        print('tot_cost: ',int(cost_charger + cost_spares + cost_elec + cost_delay ))   
        print()

        # ADD THE DATA TO THE COST LISTS        
        COST[(n_c,n_s)].append(int(cost_charger + cost_spares + cost_elec + cost_delay ))
        COST_elec[(n_c,n_s)].append(int(cost_elec))
        COST_delay[(n_c,n_s)].append(int(cost_delay ))
            

#%% GRAPH THE FINAL COSTS AS A STACKED BAR CHART

size_dates      = [(len(sb[hub][dates[i]]) , dates[i], i) for i in range(len(dates))]
size_dates.sort()

median_day      = np.where(np.array(dates_SA) == dates_MD[0])[0][0]
peak_day        = size_dates[-1][2]

fig,axs         = plt.subplots(figsize = (9,2.5), ncols = 3, nrows =2 , gridspec_kw={'height_ratios': [1,2]})
titles          = ['Expected cost', 'Median-day','Peak-day']
cost_comp       = ['chargers', 'spare batteries','electricity' , 'delays']
cost_comp_color = ['navy', 'dodgerblue', 'orangered','salmon']
hat             = ['','','/', '/']
names           = ['SA','PD','MD']
costs           = np.zeros((len(NcNs), 4, 3))

for i in range(len(NcNs)):
    ncs = NcNs[i]
    nc = ncs[0]
    ns = ncs[1]
    
    costs[i , 0 , :] = nc * c_charger
    costs[i , 1 , :] = ns * c_spare 
    
    costs[i , 2 , 0] = np.mean(COST_elec[ncs])
    costs[i , 3 , 0] = np.mean(COST_delay[ncs])
    
    costs[i , 2 , 1] = COST_elec[ncs][median_day]
    costs[i , 3 , 1] = COST_delay[ncs][median_day]
    
    costs[i , 2 , 2] = COST_elec[ncs][peak_day]
    costs[i , 3 , 2] = COST_delay[ncs][peak_day]
    

costs = costs / 1000
for i in range(len(NcNs)):
    print(names[i], '[average, median, peak day] / [chargers, spares, elec, delay,total]')
    print( costs[i,:,:])

    print(np.sum(costs[i,:,:],axis = 0))
    print(' ' )
    for comp in range(4):
        for m in range(3):
            axs[0,m].bar(i , costs[i, comp, m] , bottom = sum(costs[i, :comp , m]), color = cost_comp_color[comp], hatch = hat[comp])
            axs[1,m].bar(i , costs[i, comp, m] , bottom = sum(costs[i, :comp , m]), color = cost_comp_color[comp], hatch = hat[comp])



for i in range(3):
    #axs[i].set_box_aspect(1)
    axs[0,i].set_ylim((42,44.5))
    axs[0,i].set_yticks([42,44])
    axs[0,i].set_yticklabels([22,24])
    axs[1,i].set_ylim((0,5))
    
    
    axs[0,i].spines['bottom'].set_visible(False)
    axs[1,i].spines['top'].set_visible(False)
    axs[1,i].set_xticks(list(range(len(NcNs))))
    axs[1,i].set_xticklabels(names)
    axs[0,i].set_xticks([])
    axs[1,i].set_xlabel(titles[i])
    
    d = .5  # proportion of vertical to horizontal extent of the slanted line
    kwargs = dict(marker=[(-1, -d), (1, d)], markersize=12,
                  linestyle="none", color='k', mec='k', mew=1, clip_on=False)
    axs[0,i].plot([0, 1], [0, 0], transform=axs[0,i].transAxes, **kwargs)
    axs[1,i].plot([0, 1], [1, 1], transform=axs[1,i].transAxes, **kwargs)

    
    if i > 0:
        axs[0,i].set_yticklabels([])
        axs[1,i].set_yticklabels([])
    
for comp in range(4):
    axs[0,1].bar(0 , 0 , color = cost_comp_color[comp], label = cost_comp[comp],hatch = hat[comp])
    
axs[0,1].legend(loc = 'upper center', bbox_to_anchor = (0.5, 1.6), ncol = 4)
    

axs[1,0].set_ylabel('             Daily cost [1000€]')
plt.savefig('Output/Cost_analysis_'+ hub.IATA +'.pdf',bbox_inches = "tight")




#%% GRAPH THE FINAL SOLUTIONS AS A SCATTER PLOT PER DAY OF OPERATIONS

fig, ax     = plt.subplots(figsize = (4,3))
datesize    = [len(sb[hub][dates[i]]) for i in range(len(dates))]
mlab        = ['S(YR)',
               'S(PD) ' + dt.datetime.strftime(dates_PD[0], '%d-%m-%Y'),
               'S(MD) ' + dt.datetime.strftime(dates_MD[0], '%d-%m-%Y')]
cols        = ['navy','dodgerblue','cyan']
shape       = ['o','s','*']
for i in range(len(NcNs)):
    ncs = NcNs[i]
    ax.scatter(datesize, np.array(COST[ncs])/1000, label = mlab[i], alpha = 0.7 ,color = cols[i], marker = shape[i])
    
ax.legend()
ax.set_ylim([0,15])
ax.set_xlim([0,60])
ax.set_xlabel('Number of flights per day')
ax.set_ylabel('Daily cost [1000€]')
plt.savefig('Output/Costs_vs_NFlights_Bodo.pdf',bbox_inches = "tight")

#%% CREATE A GRAPH OF THE CHARGING SCHEDULE ON THE MEDIAN AND PEAK DAY OF OPERATIONS, 
#   GIVEN THE DIFFERENT INFRASTRUCTURE SIZES

days = {'median day' : dates_MD[0] , 'peak day': dates_PD[0]}

solutions = Hsolutions[hub]

SOLS = {d: {s:{'t': None , 'Dem': None, 'Sup':None, 'E_cons':None} for s in solutions} for d in days}

i_d = 0
for day in days:
    supply = {}
    elec_cons = {}
    for s in solutions:
        n_c = solutions[s][0]
        n_s = solutions[s][1]
        date = days[day]
        INST = BF.Create_Day_Schedule_Instance(sb, hub , date, 
                             n_c , n_s ,
                           c_delay , fl_range, 
                           bat_cap = C, pc_max = Pch , 
                           c_e_off = c_e_off , c_e_peak = c_e_peak)

        ret = INST.Solve_Model(write = 1)
                       
        T_c = ret[0] ; Rdy = ret[1] ; Dem_min_sup = ret[2] 
        Tot_chr_time = ret[3] ; cost_delay = ret[4] ;  cost_elec = ret[5] 
        
        BF.PlotChargingSchedule(INST, T_c, C/Pch * 3600, day + ', ' + s + ' solution')
        plt.savefig('Output/Charging_schedule_' + day + '_'+ s + '.pdf',bbox_inches = 'tight' )
        
        supply[s] = np.array(list(Dem_min_sup.values()))
        
        supply[s] = {i:INST.n_n + INST.n_s for i in INST.Intervals}
        for (a, i) in Rdy:
            if Rdy[(a,i)] == 1:
                supply[s][i] += 1
        
        SOLS[day][s]['Sup']     = np.array(list(supply[s].values()))
        SOLS[day][s]['E_cons']  = np.array(list(Tot_chr_time.values())) / 900 * Pch
        SOLS[day][s]['Dem']     = np.array([i.Dem for i in INST.Intervals])
        SOLS[day][s]['t']       = np.array([i.t_s for i in INST.Intervals])
        SOLS[day][s]['E_price'] = np.array([i.c_e for i in INST.Intervals])
        
    
#%% CREATE A GRAPH OF THE ELECTRICITY DEMAND AND BATTERY SURPLUS/BACKLOG ON THE MEDIAN
#   AND PEAK DAYS OF OPERATIONS

fig , axs = plt.subplots(nrows = 2 , ncols = 2, figsize = (6,4))

axs[1,0].set_xlabel('Time')
axs[1,1].set_xlabel("Time")
axs[1,1].set_xlabel("Time")
axs[0,0].set_ylabel('Battery\nsupply / backlog')
axs[1,0].set_ylabel('Electricity use [kW]')

i_d = 0
s = 'SA'
for d in days:

    t           = SOLS[d][s]['t']
    Dem         = SOLS[d][s]['Dem']
    Sup         = SOLS[d][s]['Sup']
    elec_cons   = SOLS[d][s]['E_cons']
    elec_price  = SOLS[d][s]['E_price']
    
    axs[0,i_d].plot(t,np.zeros(len(t)), color = 'black')
    axs[0,i_d].set_ylim((-5,15))
    
    axs[0,i_d].set_xlim((min(t), min(t) + dt.timedelta(days = 1,hours = 1)))
    axs[1,i_d].set_xlim((min(t), min(t) + dt.timedelta(days = 1,hours = 1)))
    axs[0,i_d].bar(t, Sup - Dem,  width = 1/96,
                    color=np.where(Sup - Dem < 0, 'crimson', 'dodgerblue'),
                    label = 'Supply - Dem ' + s )
    axs[0,i_d].set_xticklabels([])
    axs[0,i_d].grid(axis = 'x')

    if i_d > 0:
        axs[0,i_d].set_yticklabels([])
        axs[1,i_d].set_yticklabels([])
    else:
        axs[0,i_d].set_yticks([-5,0,5,10,15])
        
        
    
    axs[1,i_d].bar(t, elec_cons,width = 1/96 , color = 'dodgerblue',
                        label = 'Power ' + s)
    axs[1,i_d].set_ylim((0,1000))
    axs[0,i_d].set_title(titles[i_d + 1])
    axs[1,i_d].grid(axis = 'x')
    axs[1,i_d].set_xticklabels(axs[1,i_d].get_xticklabels(), rotation = 45)
    a2 = axs[1,i_d].twinx()
    a2.step(t, elec_price * 3600/Pch,'--', color = 'red')
    a2.set_ylim((0,0.15))
    if i_d < 1:
        a2.set_yticklabels([])
    else:
        a2.set_ylabel('Electricity price [€/kWh]',color = 'red')
        a2.legend()
    axs[1,i_d].xaxis.set_major_formatter(mdates.DateFormatter('%H:%M'))
    
    a2.tick_params(axis='y', labelcolor='red')
    
    i_d += 1

    plt.tight_layout()
    plt.savefig('Output/Figure_Daily_operations_'+ s+ '.pdf',bbox_inches = "tight")
    

